<?php 
/**
 * @package Wapple_Architect
 * @author Rich Gubby
 * @version 1.4.7
 */
/*
Plugin Name: Wapple Architect
Plugin URI: http://mobilewebjunkie.com/wordpress-mobile-plugin-install-guide-and-faq/
Description: Wapple Architect Mobile Plugin for Wordpress is a plugin that allows you to mobilize your blog in minutes. After activating this plugin visit <a href="options-general.php?page=architect.php">the settings page</a> and enter your Wapple Architect Dev Key.
Author: Rich Gubby
Version: 1.4.7
Author URI: http://mobilewebjunkie.com/
*/

/** 
 * Define some defaults - shouldn't need to change these
 * Defaults are now inserted into the DB at installation - these are left in for legacy versions
 */
// Width of images on the homepage
define('ARCHITECT_HOME_IMAGESCALE', 50);
// Quality of images on the homepage
define('ARCHITECT_HOME_IMAGEQUALITY', 95);
// Width of images on post pages
define('ARCHITECT_SINGLE_IMAGESCALE', 50);
// Quality of images on post pages
define('ARCHITECT_SINGLE_IMAGEQUALITY', 95);
// Width of images on designated pages
define('ARCHITECT_PAGE_IMAGESCALE', 50);
// Quality of images on designated pages
define('ARCHITECT_PAGE_IMAGEQUALITY', 95);
// Length of chars in post before a new page
define('ARCHITECT_SINGLE_PAGINGLENGTH', 2000);

/**
 * Debug purposes only - you shouldn't need to change these
 */ 
// Force the mobile check to always return true
define('ARCHITECT_FORCE_MOBILE', false);
// Turn a bit more debug on than is normal
define('ARCHITECT_DEBUG', false);

if(ARCHITECT_DEBUG)
{
	error_reporting(E_ALL);
	ini_set('display_errors', true);
}

if ( !defined('WP_CONTENT_DIR') )
	define( 'WP_CONTENT_DIR', ABSPATH . 'wp-content' ); // no trailing slash, full paths only - WP_CONTENT_URL is defined further down

if ( !defined('WP_PLUGIN_DIR') )
	define( 'WP_PLUGIN_DIR', WP_CONTENT_DIR . '/plugins' ); // full path, no trailing slash

if ( !defined('WP_CONTENT_URL') )
	define( 'WP_CONTENT_URL', get_option('siteurl') . '/wp-content'); // full url - WP_CONTENT_DIR is defined further up
	
if ( !defined('WP_PLUGIN_URL') )
	define( 'WP_PLUGIN_URL', WP_CONTENT_URL . '/plugins' ); // full url, no trailing slash
	
if ( !defined('WP_UPLOAD_DIR') )
	define('WP_UPLOAD_DIR', WP_CONTENT_DIR . '/uploads' );

	
	
if(!function_exists('simplexml_load_string') || (get_option('architect_devkey') == '' OR !get_option('architect_devkey')))
{
	function ArchitectInitError()
	{
		if(!isset($_REQUEST['page']) OR (isset($_REQUEST['page']) AND $_REQUEST['page'] != 'architect.php'))
		{
			echo '<div class="updated" style="background:#F24318;color:#000;border-color:#B23112;"><p>There was a problem initializing the Wapple Architect mobile plugin, please check <a href="options-general.php?page=architect.php">the Wapple Architect settings page</a> for more information</p></div>';
		}
	}
}

/**
 * Main device detection kick off function
 * - Sets up a few global variables that we're going to use later and initiates a check for mobile.
 * - If the check for mobile comes back true, amend the path to the theme
 * @return void
 */
function deviceDetection()
{
	// Only do this if we're on a non-admin page (for now!)
	if(!is_admin())
	{
		// Setup a global SOAP client so we can use the same one in the theme
		global $waplSoapClient;
		// Setup global headers so we don't have to get them twice
		global $waplHeaders;
		
		// Check if we've got SOAP and simpleXML installed
		if(class_exists('SoapClient'))
		{
			// Define whether a SOAP call is to be made
			define('ARCHITECT_DO_SOAP', true);
			define('ARCHITECT_DO_REST', false);
		} else if(function_exists('curl_init'))
		{
			// Define whether a REST call is to be made
			define('ARCHITECT_DO_SOAP', false);
			define('ARCHITECT_DO_REST', true);
		} else
		{
			return false;
		}
		if(!function_exists('simplexml_load_string'))
		{
			return false;
		}
		
		// Create the SOAP client
		if(ARCHITECT_DO_SOAP)
		{
			$waplSoapClient = __getSoapClient();
		} else
		{
			$waplSoapClient = null;
		}
		// Get device / browser header information 
		$waplHeaders = __getHeaders();
		
		// If __testForMobile() comes back true, amend template path
		if(__testForMobile())
		{
			// Load the correct file depending on the URL (copied from template-loader.php)
			$template = __templateLoader();
			
			$file = WP_PLUGIN_DIR.DIRECTORY_SEPARATOR.get_wapl_plugin_base().DIRECTORY_SEPARATOR.'theme'.DIRECTORY_SEPARATOR.$template;
			if(file_exists($file))
			{
				include($file);
				exit(0);
			}
		}
	}
}

/**
 * Work out which theme file to use
 * @return string
 */
function __templateLoader()
{
	if ( is_404()) {
		return '404.php';
	} else if ( is_search()) {
		return 'search.php';
	} else if ( is_home()) {
		return 'index.php';
	} else if ( is_single()) {
		return 'single.php';
	} else if ( is_page()) {
		return 'page.php';
	} else if ( is_archive()) {
		return 'archive.php';
	} else if (file_exists(WP_PLUGIN_DIR.DIRECTORY_SEPARATOR.get_wapl_plugin_base().DIRECTORY_SEPARATOR.'theme'.DIRECTORY_SEPARATOR.'index.php' )) {
		return 'index.php';
	}
}

/**
 * Perform a device detection check
 * @return boolean
 */
function __testForMobile()
{
	global $waplSoapClient;
	global $waplHeaders;
	
	if(isset($_REQUEST['mobile']) && $_REQUEST['mobile'] == 0)
	{
		setcookie('isMobile', "", time()-3600, '/');
		setcookie('isMobile', "0", time()+3600, '/');
		return false;
	} else if(isset($_REQUEST['mobile']) && $_REQUEST['mobile'] == 1)
	{
		setcookie('isMobile', "", time()-3600, '/');
		setcookie('isMobile', "1", time()+3600, '/');
		return true;
	}
	
	if(!get_option('architect_devkey'))
	{
		return false;
	}
	
	// Check for existence of a cookie
	if(isset($_COOKIE['isMobile']) AND $_COOKIE['isMobile'] == true)
	{
		return true;
	} else if(isset($_COOKIE['isMobile']) AND $_COOKIE['isMobile'] == false)
	{
		return false;
	} else
	{
		if(ARCHITECT_DO_SOAP)
		{
			// Test for mobile
			$params = array(
				'devKey' => get_option('architect_devkey'),
				'deviceHeaders' => $waplHeaders
			);
		
			$result = $waplSoapClient->isMobileDevice($params);
			if(is_soap_fault($result))
			{
				// There is a SOAP error! probably a dev key error
				return false;
			}  else
			{
				if($result)
				{
					// Set a cookie to remember the outcome!
					setcookie('isMobile', true, time()+3600, '/');
					return true;
				} else
				{
					setcookie('isMobile', false, time()+3600, '/');
					return false;
				}
			}
		} else if(ARCHITECT_DO_REST)
		{
			$postfields = array(
				'devKey' => get_option('architect_devkey'),
				'headers' => $waplHeaders
			);
			
			$c = curl_init();
			curl_setopt($c, CURLOPT_URL, 'http://webservices.wapple.net/isMobileDevice.php');
			curl_setopt($c, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($c, CURLOPT_POST, 1);
			curl_setopt($c, CURLOPT_POSTFIELDS, $postfields);
			
			$result = curl_exec($c);
			curl_close($c);
			if($result == 1)
			{
				setcookie('isMobile', true, time()+3600, '/');
				return true;
			} else
			{
				setcookie('isMobile', $result, time()+3600, '/');
				return false;
			}
		}
		
	}
}

/**
 * Get a SOAP client
 * @return mixed
 */
function __getSoapClient()
{
	if(ARCHITECT_DO_SOAP)
	{
		return new SoapClient('http://webservices.wapple.net/wapl.wsdl', array('exceptions' => 0));
	} else
	{
		return false;
	}	
}

/**
 * Build device header array that SOAP can use
 * @return array
 */
function __getHeaders()
{
	if(ARCHITECT_DO_SOAP)
	{
		$headers = array();
		foreach($_SERVER as $key => $val)
		{
			$headers[] = array('name' => $key, 'value' => $val);
		}
		return $headers;
	} else
	{
		$headers = '';
		foreach($_SERVER as $key => $val)
		{
			$headers .= $key.':'.$val.'|';
		}
		return $headers;
	}
}

// Add the deviceDetection function to the template_redirect filter
add_action('template_redirect', 'deviceDetection');
// Add architect options to admin menu
add_action('admin_menu', 'add_architect_options_page');
// Add notice to WordPress if we have an architect error
if(function_exists('ArchitectInitError'))
{
	add_action('admin_head','ArchitectInitError');
}

/**
 * Add architect admin options
 * @return void
 */
function add_architect_options_page()
{
	add_options_page('Wapple Architect Settings', 'Wapple Architect', 8, basename(__FILE__), 'architect_options_page');
}

function architect_save_option($option)
{
	if($_POST[$option] != get_option($option))
	{
		update_option($option, $_POST[$option]);
		return true;
	}
}

/**
 * Display architect options and save when requested
 * @return void
 */
function architect_options_page()
{
	echo '<div class="wrap">';
	
	if (isset($_POST['info_update'])) 
	{
		$updateOption = false;

		// Save Dev Key
		if(architect_save_option('architect_devkey')) $updateOption = true;
		
		// Save the show blog description
		if(architect_save_option('architect_show_blogdesc')) $updateOption = true;
		
		// Save the mobile blog title
		if(architect_save_option('architect_mobile_blogtitle')) $updateOption = true;
		
		// Save the mobile blog description
		if(architect_save_option('architect_mobile_blogdesc')) $updateOption = true;

		// Save META tags
		if(architect_save_option('architect_use_meta')) $updateOption = true;
				
		// Save META keywords
		if(architect_save_option('architect_meta_keywords')) $updateOption = true;
		
		// Save use header image
		if(architect_save_option('architect_use_headerimage')) $updateOption = true;

		// Save actual header image
		if(isset($_FILES['architect_headerimage']))
		{
			$pathToHeader = WP_UPLOAD_DIR.DIRECTORY_SEPARATOR.get_wapl_plugin_base().DIRECTORY_SEPARATOR.'images'.DIRECTORY_SEPARATOR.'header'.DIRECTORY_SEPARATOR;
			architect_check_path($pathToHeader);
			if(move_uploaded_file($_FILES['architect_headerimage']['tmp_name'], $pathToHeader.$_FILES['architect_headerimage']['name']))
			{
				update_option('architect_headerimage', $_FILES['architect_headerimage']['name']);
				$updateOption = true;
			}
		}
		// Save header image BG transparency col
		if(architect_save_option('architect_headerimage_transparency_colour')) $updateOption = true;

		// Is the header image a link back to the homepage
		if(architect_save_option('architect_headerimagelinktohome')) $updateOption = true;
				
		// Save use footer image
		if(architect_save_option('architect_use_footerimage')) $updateOption = true;

		// Save actual footer image
		if(isset($_FILES['architect_footerimage']))
		{
			$pathToFooter = WP_UPLOAD_DIR.DIRECTORY_SEPARATOR.get_wapl_plugin_base().DIRECTORY_SEPARATOR.'images'.DIRECTORY_SEPARATOR.'footer'.DIRECTORY_SEPARATOR;
			architect_check_path($pathToFooter);
			if(move_uploaded_file($_FILES['architect_footerimage']['tmp_name'], $pathToFooter.$_FILES['architect_footerimage']['name']))
			{
				update_option('architect_footerimage', $_FILES['architect_footerimage']['name']);
				$updateOption = true;
			}
		}
		// Save footer image BG transparency col
		if(architect_save_option('architect_footerimage_transparency_colour')) $updateOption = true;
		
		// Show footer text
		if(architect_save_option('architect_showfootertext')) $updateOption = true;
		
		// What is the footer text
		if(architect_save_option('architect_footertext')) $updateOption = true;
		
		// Save show menu at top
		if(architect_save_option('architect_showmenuattop')) $updateOption = true;
		
		// Save show menu at bottom
		if(architect_save_option('architect_showmenuatbottom')) $updateOption = true;
		
		// Save switch to desktop
		if(architect_save_option('architect_showswitchtodesktop')) $updateOption = true;
		
		// Save show author
		if(architect_save_option('architect_showauthor')) $updateOption = true;
		
		// Save try and do forms
		if(architect_save_option('architect_doform')) $updateOption = true;
		
		// Home page
		
		// Save show excerpt on home page
		if(architect_save_option('architect_home_showexcerpt')) $updateOption = true;
		
		// Save home page excerpt override
		if(architect_save_option('architect_home_excerpt_override')) $updateOption = true;
		
		// Save home page show categories
		if(architect_save_option('architect_home_showcategories')) $updateOption = true;
		
		// Save home page image scale
		if(architect_save_option('architect_home_imagescale')) $updateOption = true;
		
		// Save home page image quality
		if(architect_save_option('architect_home_imagequality')) $updateOption = true;
		
		// Save home page transparency colour
		if(architect_save_option('architect_home_transcol')) $updateOption = true;
		
		// Save number of posts on home page
		if(architect_save_option('architect_home_posts_per_page')) $updateOption = true;
		
		// Save show more link
		if(architect_save_option('architect_home_showmore')) $updateOption = true;
		
		// Post Page
		
		// Save use post header image
		if(architect_save_option('architect_single_use_headerimage')) $updateOption = true;
		
		// Save post header image
		if(isset($_FILES['architect_single_headerimage']))
		{
			$pathToPostHeader = WP_UPLOAD_DIR.DIRECTORY_SEPARATOR.get_wapl_plugin_base().DIRECTORY_SEPARATOR.'images'.DIRECTORY_SEPARATOR.'postHeader'.DIRECTORY_SEPARATOR;
			architect_check_path($pathToPostHeader);
			if(move_uploaded_file($_FILES['architect_single_headerimage']['tmp_name'], $pathToPostHeader.$_FILES['architect_single_headerimage']['name']))
			{
				update_option('architect_single_headerimage', $_FILES['architect_single_headerimage']['name']);
				$updateOption = true;
			}
		}
		
		// Save post do paging
		if(architect_save_option('architect_single_dopaging')) $updateOption = true;
		
		// Save post paging length
		if(architect_save_option('architect_single_paginglength')) $updateOption = true;
		
		// Save show home link
		if(architect_save_option('architect_single_showhome')) $updateOption = true;
		
		// Save show post next link
		if(architect_save_option('architect_single_shownext')) $updateOption = true;
		
		// Save show tags
		if(architect_save_option('architect_single_showtags')) $updateOption = true;
		
		// Save post image scale
		if(architect_save_option('architect_single_imagescale')) $updateOption = true;
		
		// Save post image quality
		if(architect_save_option('architect_single_imagequality')) $updateOption = true;
		
		// Save post image transcol
		if(architect_save_option('architect_single_transcol')) $updateOption = true;
		
		// Save post response format
		if(architect_save_option('architect_single_responseformat')) $updateOption = true;
		
		// Save allow post comments
		if(architect_save_option('architect_single_allowcomments')) $updateOption = true;
		
		// Save show post comments
		if(architect_save_option('architect_single_showcomments')) $updateOption = true;
		
		// Save read more on post
		if(architect_save_option('architect_single_nextpage')) $updateOption = true;
		
		// Save read less on post
		if(architect_save_option('architect_single_prevpage')) $updateOption = true;

		// Save text for previous post
		if(architect_save_option('architect_single_previouspost')) $updateOption = true;
		
		// Save text for next post
		if(architect_save_option('architect_single_nextpost')) $updateOption = true;
		
		// Save text for home post
		if(architect_save_option('architect_single_homepost')) $updateOption = true;
		
		// Page Settings
		
		// Save page image scale
		if(architect_save_option('architect_page_imagescale')) $updateOption = true;
		
		// Save page image quality
		if(architect_save_option('architect_page_imagequality')) $updateOption = true;
		
		// Save page  image transcol
		if(architect_save_option('architect_page_transcol')) $updateOption = true;
		
		// Search options
		
		// Save home show search
		if(architect_save_option('architect_home_showsearch')) $updateOption = true;
		
		// Save search page show tags
		if(architect_save_option('architect_search_showtags')) $updateOption = true;
		
		// Save search page show categories
		if(architect_save_option('architect_search_showcategories')) $updateOption = true;
		
		// Save Mobile stylesheet
		$file = WP_UPLOAD_DIR.DIRECTORY_SEPARATOR.get_wapl_plugin_base().DIRECTORY_SEPARATOR.'css'.DIRECTORY_SEPARATOR.'style.css';
		architect_check_path(WP_UPLOAD_DIR.DIRECTORY_SEPARATOR.get_wapl_plugin_base().DIRECTORY_SEPARATOR.'css'.DIRECTORY_SEPARATOR);
		
		if(is_readable($file))
		{
			$stylesheet = file_get_contents($file);
		} else
		{
			$stylesheet = '';
		}
		
		if(!file_exists($file))
		{
			// Create file if it doesn't exist
			@file_put_contents($file, "");
		}
		
		if(is_writable($file))
		{
			if(stripslashes($_POST['architect_stylesheet']) != $stylesheet)
			{
				if(file_put_contents($file, stripslashes($_POST['architect_stylesheet'])))
				{
					$updateOption = true;
				}
			}
		} else
		{
			echo '<div class="updated" style="background:#F24318;color:#000;border-color:#B23112;"><p>"'.$file.'" is not writable - change the permissions to save the stylesheet!</p></div>';
		}
		
		// Save custom stylesheet
		$custom_file = WP_UPLOAD_DIR.DIRECTORY_SEPARATOR.get_wapl_plugin_base().DIRECTORY_SEPARATOR.'css'.DIRECTORY_SEPARATOR.'custom_style.css';
		
		if(is_readable($custom_file))
		{
			$custom_stylesheet = file_get_contents($custom_file);
		} else
		{
			$custom_stylesheet = '';
		}
		
		if(!file_exists($custom_file))
		{
			// Create file if it doesn't exist
			@file_put_contents($custom_file, "");
		}
		
		if(is_writable($custom_file))
		{
			if(stripslashes($_POST['architect_custom_stylesheet']) != $custom_stylesheet)
			{
				if(file_put_contents($custom_file, stripslashes($_POST['architect_custom_stylesheet'])))
				{
					$updateOption = true;
				}
			}
		} else
		{
			echo '<div class="updated" style="background:#F24318;color:#000;border-color:#B23112;"><p>"'.$custom_file.'" is not writable - change the permissions to save the stylesheet!</p></div>';
		}
		
		if($updateOption)
		{
			echo "<div class='updated fade'><p><strong>Wapple Architect settings saved.</strong></p></div>";
		}
	}
	
	if(get_option('architect_devkey') == '' OR !get_option('architect_devkey'))
	{
		echo '<div class="updated" style="background:#F24318;color:#000;border-color:#B23112;"><p><strong>You haven\'t saved your free Wapple Architect Dev Key, get one from here: <a href="http://wapple.net/register/free-developer-program-for-coding-mobile-web.htm" style="color:#000;">http://wapple.net/register/free-developer-program-for-coding-mobile-web.htm</a></strong></p></div>';
	}
	if(!function_exists('simplexml_load_string'))
	{
		echo '<div class="updated" style="background:#F24318;color:#000;border-color:#B23112;"><p>You do not have simpleXML installed, this plugin needs simpleXML in order to parse XML returned from Wapple\'s web services</p></div>';
	}
	
	echo '<form method="post" action="options-general.php?page=architect.php" enctype="multipart/form-data">';
	echo architect_admin_header('2', 'Wapple Architect Settings');
	
	echo '<p><em>Wapple Architect WordPress plugin is brought to you by Rich Gubby - <a href="http://mobilewebjunkie.com">http://mobilewebjunkie.com</a></em></p>';

	
	echo '
	<div id="architectTabs">
		<ul class="architectclearfix">
		<li id="architectTabBasic" class="selected"><a href="#basic" onclick="architectSwitchTab(\'Basic\');return false;">Basic</a></li>
		<li id="architectTabHome"><a href="#home" onclick="architectSwitchTab(\'Home\');return false;">Home Page</a></li>
		<li id="architectTabPosts"><a href="#posts" onclick="architectSwitchTab(\'Posts\');return false;">Posts Page</a></li>
		<li id="architectTabPages"><a href="#pages" onclick="architectSwitchTab(\'Pages\');return false;">Pages Page</a></li>
		<li id="architectTabSearch"><a href="#search" onclick="architectSwitchTab(\'Search\');return false;">Search Results</a></li>
		<li id="architectTabTheme"><a href="#theme" onclick="architectSwitchTab(\'Theme\');return false;">Mobile Theme</a></li>
		</ul>
	</div>
	';
	
	// Basic Settings
	require_once('admin'.DIRECTORY_SEPARATOR.'options-basic.php');
	
	// Home page settings
	require_once('admin'.DIRECTORY_SEPARATOR.'options-home.php');
	
	// Posts Page Settings
	require_once('admin'.DIRECTORY_SEPARATOR.'options-posts.php');
	
	// Pages Settings
	require_once('admin'.DIRECTORY_SEPARATOR.'options-pages.php');
	
	// Search Results Settings
	require_once('admin'.DIRECTORY_SEPARATOR.'options-search.php');
	
	// Mobile Theme Settings
	require_once('admin'.DIRECTORY_SEPARATOR.'options-theme.php');
	
	echo '<table class="form-table" cellspacing="2" cellpadding="5" width="100%"><tr><td><p class="submit"><input class="button-primary" type="submit" name="info_update" value="Save Changes" /></p></td></tr></table>';
	echo '</form></div>';
}

function architect_add_headerimage()
{
	echo '<input type="file" name="architect_add_headerimage" id="architect_add_headerimage" />';
}

if(!function_exists('get_wapl_plugin_base'))
{
	function get_wapl_plugin_base()
	{
		// Fix non conforming directory separators!
		$opposite = array('/' => '\\', '\\' => '/');
		$base = str_replace($opposite[DIRECTORY_SEPARATOR], DIRECTORY_SEPARATOR, plugin_basename(__FILE__));
		
		$baseExplode = explode(DIRECTORY_SEPARATOR, $base);
		return $baseExplode[0];
	}
}

function architect_check_path($path)
{
	$path = str_replace(WP_UPLOAD_DIR, '', $path);
	
	$dirExplode = explode(DIRECTORY_SEPARATOR, $path);
	$pathToCheck = WP_UPLOAD_DIR;
	foreach($dirExplode as $key => $val)
	{
		if(!file_exists($pathToCheck))
		{
			@mkdir($pathToCheck, 0777);
		}
		$pathToCheck .= $val.DIRECTORY_SEPARATOR;
	}
}

if(!function_exists('architect_admin_header'))
{
	function architect_admin_header($h, $value)
	{
		return '<tr><td colspan="2"><h'.$h.' style="margin:0;font-size:1.3em;">'.$value.'</h'.$h.'></td></tr>';
	}
}
if(!function_exists('architect_table_start'))
{
	function architect_table_start()
	{
		return '<table class="form-table" cellspacing="2" cellpadding="5" style="margin-top:0;padding-top:5px;margin-bottom:0px;border:solid 1px #AAAAAA;border-width: 0 1px 1px;">';
	}
}
if(!function_exists('architect_admin_option'))
{
	function architect_admin_option($type, $options = array())
	{
		$string  = '<tr>';
		$string .= '<th width="30%" valign="top" style="padding-top: 10px;">';
		
		if(isset($options['name']))
		{
			$string .= '<label for="'.$options['name'].'">'.$options['label'].': </label>';
		}
		$string .= '</th><td>';
		
		switch($type)
		{
			case 'input' : 
				if(!isset($options['size']))
				{
					$options['size'] = 40;
				}
				if(isset($options['before']) AND $options['before'] != '')
				{
					$string .= $options['before'];
				}
				$string .= '<input';
				
				if($options['size'] == 40)
				{
					$string .= ' class="regular-text"';
				}
				$string .= ' style="margin-right:10px;" size="'.$options['size'].'" type="text" name="'.$options['name'].'" id="'.$options['name'].'" value="'.$options['value'].'" />';
				
				if(isset($options['after']) AND $options['after'] != '')
				{
					$string .= $options['after'];
				}
				break;
			case 'select' :
				$string .= '<select name="'.$options['name'].'" style="margin-right:10px;">';
				foreach($options['options'] as $key => $val)
				{
					$string .= '<option value="'.$key.'"';
				
					if($key == $options['value'])
					{
						$string .= ' selected="selected"';
					}
					
					$string .= '>'.$val.'</option>';
				}
				$string .= '</select>';
				break;
			case 'file' :
				$string .= '<input type="file" name="'.$options['name'].'" id="'.$options['name'].'" style="margin-right:10px;" />';
				break;
			case 'image' :
				$string .= '<img src="'.$options['src'].'" alt="'.$options['alt'].'" style="margin-right:10px;" />';
				break;
			case 'textarea' :
				if(!isset($options['rows']))
				{
					$options['rows'] = 25;
				}
				if(!isset($options['cols']))
				{
					$options['cols'] = 70;
				}
				$string .= '<textarea style="margin-right:10px;" id="'.$options['name'].'" name="'.$options['name'].'" rows="'.$options['rows'].'" cols="'.$options['cols'].'">'.$options['value'].'</textarea>';
				break;
		}
		
		if(isset($options['description']))
		{
			$string .= '<span class="description">'.$options['description'].'</span>';
		}
		
		$string .= '</td></tr>';
		return $string;
	}
}

register_activation_hook(__FILE__, 'architect_install');

function architect_install()
{
	if(!$blog_id = get_option('blog_id'))
	{
		$blog_id = 0;
	}
	
	global $wpdb;
	
	$sql = '
	INSERT INTO `'.$wpdb->prefix.'options` 
	(`blog_id`, `option_name`, `option_value`, `autoload`) 
	VALUES
	('.$blog_id.', \'architect_headerimagelinktohome\', \'1\', \'yes\'),
	('.$blog_id.', \'architect_use_meta\', \'1\', \'yes\'),
	('.$blog_id.', \'architect_home_imagescale\', \'50\', \'yes\'),
	('.$blog_id.', \'architect_home_imagequality\', \'95\', \'yes\'),
	('.$blog_id.', \'architect_single_imagescale\', \'50\', \'yes\'),
	('.$blog_id.', \'architect_single_imagequality\', \'95\', \'yes\'),
	('.$blog_id.', \'architect_page_imagescale\', \'50\', \'yes\'),
	('.$blog_id.', \'architect_page_imagequality\', \'95\', \'yes\'),
	('.$blog_id.', \'architect_single_paginglength\', \'2000\', \'yes\'),
	('.$blog_id.', \'architect_single_allowcomments\', \'1\', \'yes\'),
	('.$blog_id.', \'architect_single_showcomments\', \'1\', \'yes\'),
	('.$blog_id.', \'architect_single_dopaging\', \'1\', \'yes\'),
	('.$blog_id.', \'architect_showfootertext\', \'1\', \'yes\'),
	('.$blog_id.', \'architect_single_showhome\', \'1\', \'yes\'),
	('.$blog_id.', \'architect_single_nextpage\', \'more\', \'yes\'),
	('.$blog_id.', \'architect_single_prevpage\', \'back\', \'yes\'),
	('.$blog_id.', \'architect_single_previouspost\', \'previous post\', \'yes\'),
	('.$blog_id.', \'architect_single_nextpost\', \'next post\', \'yes\'),
	('.$blog_id.', \'architect_single_homepost\', \'home\', \'yes\'),
	('.$blog_id.', \'architect_single_shownext\', \'1\', \'yes\'),
	('.$blog_id.', \'architect_single_showtags\', \'1\', \'yes\'),
	('.$blog_id.', \'architect_home_posts_per_page\', \''.get_option('posts_per_page').'\', \'yes\'),
	('.$blog_id.', \'architect_home_excerpt_override\', \'300\', \'yes\'),
	('.$blog_id.', \'architect_showfootertext\', \'Powered by [url=http://wapple.net]Wapple Architect[/url]\', \'yes\')';
	$result = $wpdb->query($sql);
	
	// Check a few paths exist
	$file = WP_UPLOAD_DIR.DIRECTORY_SEPARATOR.get_wapl_plugin_base().DIRECTORY_SEPARATOR.'images'.DIRECTORY_SEPARATOR.'header'.DIRECTORY_SEPARATOR;
	architect_check_path($file);
	$file = WP_UPLOAD_DIR.DIRECTORY_SEPARATOR.get_wapl_plugin_base().DIRECTORY_SEPARATOR.'images'.DIRECTORY_SEPARATOR.'footer'.DIRECTORY_SEPARATOR;
	architect_check_path($file);
	$file = WP_UPLOAD_DIR.DIRECTORY_SEPARATOR.get_wapl_plugin_base().DIRECTORY_SEPARATOR.'images'.DIRECTORY_SEPARATOR.'postHeader'.DIRECTORY_SEPARATOR;
	architect_check_path($file);
	$file = WP_UPLOAD_DIR.DIRECTORY_SEPARATOR.get_wapl_plugin_base().DIRECTORY_SEPARATOR.'css'.DIRECTORY_SEPARATOR;
	architect_check_path($file);
	$file = WP_UPLOAD_DIR.DIRECTORY_SEPARATOR.get_wapl_plugin_base().DIRECTORY_SEPARATOR.'css'.DIRECTORY_SEPARATOR.'style.css';
	architect_check_path($file);
	$file = WP_UPLOAD_DIR.DIRECTORY_SEPARATOR.get_wapl_plugin_base().DIRECTORY_SEPARATOR.'css'.DIRECTORY_SEPARATOR.'custom_style.css';
	architect_check_path($file);
	
	// Copy stylesheet to default location
	$file = WP_UPLOAD_DIR.DIRECTORY_SEPARATOR.get_wapl_plugin_base().DIRECTORY_SEPARATOR.'css'.DIRECTORY_SEPARATOR.'style.css';
	
	if(!file_exists($file))
	{
		@file_put_contents($file, "");
		if(is_writable($file))
		{
			file_put_contents($file, file_get_contents(WP_PLUGIN_DIR.DIRECTORY_SEPARATOR.get_wapl_plugin_base().DIRECTORY_SEPARATOR.'theme'.DIRECTORY_SEPARATOR.'style.css'));
		}
	}
		
	$file = WP_UPLOAD_DIR.DIRECTORY_SEPARATOR.get_wapl_plugin_base().DIRECTORY_SEPARATOR.'css'.DIRECTORY_SEPARATOR.'custom_style.css';
	if(!file_exists($file))
	{
		@file_put_contents($file, "");
		if(is_writable($file))
		{
			file_put_contents($file, file_get_contents(WP_PLUGIN_DIR.DIRECTORY_SEPARATOR.get_wapl_plugin_base().DIRECTORY_SEPARATOR.'theme'.DIRECTORY_SEPARATOR.'custom_style.css'));
		}
	}
}

add_filter('plugin_row_meta', 'architect_filter_plugin_links', 10, 2);

// 
function architect_filter_plugin_links($links, $file)
{
	if ( $file == plugin_basename(__FILE__) )
	{
		$links[] = '<a href="options-general.php?page=architect.php">' . __('Settings') . '</a>';
	}
	
	return $links;
}

if(!function_exists('architect_curPageURL'))
{
	function architect_curPageURL() 
	{
		$pageURL = 'http';
		if (isset($_SERVER["HTTPS"]) AND $_SERVER["HTTPS"] == "on") {$pageURL .= "s";}
		$pageURL .= "://";
		if ($_SERVER["SERVER_PORT"] != "80") {
			$pageURL .= $_SERVER["SERVER_NAME"].":".$_SERVER["SERVER_PORT"].$_SERVER["REQUEST_URI"];
		} else 
		{
			$pageURL .= $_SERVER["SERVER_NAME"].$_SERVER["REQUEST_URI"];
		}
		return $pageURL;
	}
}

if(!function_exists('architect_buildUrl'))
{
	function architect_buildUrl($url, $options = array())
	{
		unset($_GET['mobile']);
		
		if(!isset($options['mobile']))
		{
			$options['mobile'] = 1;
		}
		$parsedUrl = parse_url($url);
		
		$return = $parsedUrl['scheme'].'://'.$parsedUrl['host'].$parsedUrl['path'];
		
		$i = 0;
		foreach($_GET as $key => $val)
		{
			if($i == 0)
			{
				$return .= '?';
			} else
			{
				$return .= '&';
			}
			
			$return .= $key.'='.$val;
			$i++;
		}
		foreach($options as $key => $val)
		{
			if($i == 0)
			{
				$return .= '?';
			} else
			{
				$return .= '&';
			}
			
			$return .= $key.'='.$val;
			$i++;
		}

		return htmlspecialchars($return);
	}
}

function architect_web_footer()
{
	echo '<div id="deviceSwitcher">View in: <a href="'.architect_buildUrl(architect_curPageURL(), array('mobile' => 1)).'">Mobile</a> | Standard</div>';
}
if(get_option('architect_showswitchtodesktop'))
{
	add_action('wp_footer', 'architect_web_footer');
}

function architect_register_head()
{
	$siteurl = get_option('siteurl');
	$url = $siteurl . '/wp-content/plugins/' . basename(dirname(__FILE__)) . '/admin/architect.css';
	echo "<link rel=\"stylesheet\" type=\"text/css\" href=\"".$url."\" />\n";
	$url = $siteurl . '/wp-content/plugins/' . basename(dirname(__FILE__)) . '/admin/architect.js';
	echo "<script type=\"text/javascript\" src=\"".$url."\"></script>\n";
	$url = $siteurl . '/wp-content/plugins/' . basename(dirname(__FILE__)) . '/admin/prototype.js';
	echo '<script type="text/javascript" src="'.$url.'"></script>';
	
}
add_action('admin_head', 'architect_register_head');

if(!function_exists('jquery_no_conflict_follows_jquery'))
{
	function jquery_no_conflict_follows_jquery( $js_array ) { if ( false === $jquery = array_search( 'jquery', $js_array ) ) return $js_array; if ( false === $jquery_no_conflict = array_search( 'jquery_no_conflict', $js_array ) ) return $js_array; if ( $jquery_no_conflict == $jquery + 1 ) return $js_array; array_splice( $js_array, $jquery + 1, 0, 'jquery_no_conflict' ); unset($js_array[$jquery_no_conflict + ($jquery_no_conflict < $jquery ? 0 : 1)]); return $js_array; }
} 
add_filter( 'print_scripts_array', 'jquery_no_conflict_follows_jquery' );
?>
